from trafilatura import bare_extraction
import hashlib

def quick_content(html_text):
    content = bare_extraction(html_text)
    if content==None:
        return ""
    text = content["text"]
    return text

def get_md5_str(text):
    unique_id = hashlib.md5(text.encode())
    page_id = str(unique_id.hexdigest())
    return page_id


import os

from opinionx.tfidf import *

# current_path = os.path.dirname(os.path.realpath(__file__))

def run_tfidf_shell(input_folder,output_folder, user_dict_path="", font_path="",is_html=True):
    # data
    # source_csv_file = f"{current_path}/news_list_chinese.csv"  # contain the FileId field in the csv file
    # raw_data_folder = f"{current_path}/raw_data"  # contain txt files with name starting with FileId, e.g. xxxx-xxxx-xxxx.txt
    list_files=[]
    for file in os.listdir(input_folder):
        list_files.append(os.path.join(input_folder,file))

    if not os.path.exists(output_folder):
        os.mkdir(output_folder)

    # custom dictionary
    keywords_path = f"{output_folder}/keywords_user.pickle"
    # keyword_file_list = "cnki_data.txt;cnki_data1.txt;cnki_data2.txt"
    if user_dict_path!="":
        keyword_file_list_folder = user_dict_path
        keyword_flist=[]
        for file in os.listdir(keyword_file_list_folder):
            keyword_flist.append(file)
        keyword_file_list=';'.join(keyword_flist)
    else:
        keyword_file_list_folder=""
        keyword_file_list=""


    # corpus data path to save
    save_corpus_path = f"{output_folder}/corpus.pickle"

    # export keywords path
    save_freq_filepath = f"{output_folder}/keywords_freq.pickle"
    save_tfidf_path = f"{output_folder}/keywords_tfidf.pickle"

    run_tfidf_model(list_files,keywords_path,keyword_file_list,keyword_file_list_folder,save_corpus_path,save_freq_filepath,save_tfidf_path,is_html=is_html)

    # export keywords
    export_keywords(save_freq_filepath,save_tfidf_path,f"{output_folder}/keywords_freq.csv",f"{output_folder}/keywords_tfidf.csv")

    if keyword_file_list!="" and keyword_file_list_folder!="":
        export_keywords_dictionary(keyword_file_list, keyword_file_list_folder, f"{output_folder}/keywords_user.csv")

    # create wordcloud
    run_wordcloud(save_corpus_path, f"{output_folder}/word_cloud_auto.jpg",font_path=font_path)




from gensim import corpora, models
import gensim
from quickcsv.file import *
import jieba

def lda_shell(
        csv_path,
        id_field="file_id",
        text_field="text",
# ============ begin configure ====================
    NUM_TOPICS = 6,
    NUM_WORDS = 20,
    FIG_V_NUM = 2,
    FIG_H_NUM = 3,
    WC_MAX_WORDS = 20,
    NUM_PASS = 5,
    # ============ end configure ======================
        list_dictionary_files=None,
        stopwords_path="",
        font_path='utils/fonts/SimHei.ttf',
        output_figure_path=""
):

    if list_dictionary_files!=None:
        for file in list_dictionary_files:
            jieba.load_userdict(file)

    list_item = qc_read(csv_path)
    dict_doc = {}
    list_result = []
    for item in list_item:
        file_id = item[id_field]
        text = item[text_field]
        if file_id not in dict_doc.keys():
            dict_doc[file_id] = [text]
        else:
            dict_doc[file_id].append(text)
    list_doc = []
    for k in dict_doc.keys():
        list_doc.append('\n'.join(dict_doc[k]))

    print("document number: ", len(list_doc))

    # qc_write("results/result_expert.csv",list_result)
    stopwords=[]
    if stopwords_path!="":
        stopwords = [w.strip() for w in open(stopwords_path, 'r', encoding='utf-8').readlines()
                 if w.strip() != ""]


    # load data
    # dict_dataset=pickle.load(open("datasets/weibo_vae_dataset_prepared_with_domain.pickle", "rb"))

    # compile sample documents into a list
    # doc_set = [doc_a, doc_b, doc_c, doc_d, doc_e]
    import jieba.posseg as pseg
    doc_set = []
    for doc in list_doc:
        # list_words=jieba.cut(doc,cut_all=False)
        list_words = pseg.cut(doc)
        list_w = []
        for w, f in list_words:
            if 'n' in f:
                if w not in stopwords:
                    list_w.append(w)
        # print(list_w)
        doc_set.append(list_w)

    # list for tokenized documents in loop
    texts = []

    # loop through document list
    for tokens in doc_set:
        # clean and tokenize document string

        # stem tokens
        # stemmed_tokens = [p_stemmer.stem(i) for i in tokens]

        # add tokens to list
        texts.append(tokens)

    # turn our tokenized documents into a id <-> term dictionary
    dictionary = corpora.Dictionary(texts)

    # convert tokenized documents into a document-term matrix
    corpus = [dictionary.doc2bow(text) for text in texts]

    # generate LDA model
    ldamodel = gensim.models.ldamodel.LdaModel(corpus, num_topics=NUM_TOPICS, id2word=dictionary, passes=NUM_PASS)

    # print keywords
    topics = ldamodel.print_topics(num_words=NUM_WORDS, num_topics=NUM_TOPICS)
    for topic in topics:
        print(topic)

    # 1. Wordcloud of Top N words in each topic
    from matplotlib import pyplot as plt
    from wordcloud import WordCloud, STOPWORDS
    import matplotlib.colors as mcolors
    import matplotlib
    matplotlib.rcParams['font.family'] = 'SimHei'

    cols = [color for name, color in mcolors.TABLEAU_COLORS.items()]  # more colors: 'mcolors.XKCD_COLORS'

    cloud = WordCloud(
        background_color='white',
        width=2500,
        height=1800,
        max_words=WC_MAX_WORDS,
        colormap='tab10',
        color_func=lambda *args, **kwargs: cols[i],
        prefer_horizontal=1.0, font_path=font_path)

    topics = ldamodel.show_topics(formatted=False, num_topics=NUM_TOPICS, num_words=NUM_WORDS)

    fig, axes = plt.subplots(FIG_V_NUM, FIG_H_NUM, figsize=(10, 10), sharex=True, sharey=True)

    for i, ax in enumerate(axes.flatten()):
        fig.add_subplot(ax)
        topic_words = dict(topics[i][1])
        cloud.generate_from_frequencies(topic_words, max_font_size=300)
        plt.gca().imshow(cloud)
        plt.gca().set_title('Topic ' + str((i + 1)), fontdict=dict(size=16))
        plt.gca().axis('off')

    plt.subplots_adjust(wspace=0, hspace=0)
    plt.axis('off')
    plt.margins(x=0, y=0)
    plt.tight_layout()
    if output_figure_path!="":
        plt.savefig(output_figure_path, dpi=600)

    plt.show()


import jieba
from nerkit.StanzaApi import StanzaWrapper
from quickcsv.file import quick_save_csv

def getStanzaWrapper(**kwargs):
    return StanzaWrapper(**kwargs)

def get_opinion(text,user_dictionary_path="",opinion_words='',lang='zh',sw=None):
    if sw==None:
        sw=StanzaWrapper()

    if user_dictionary_path!="":
        jieba.load_userdict(user_dictionary_path)
    # opinion_words=['表示','认为','说','介绍','提出','透露','指出','强调','：']
    if len(text)<30:
        return []
    list_sentence=sw.tokenize_sentence(text,lang=lang)
    list_result=[]
    list_seg=[]
    list_ner=[]
    for sentence in list_sentence:
        # print(sentence)
        if len(sentence)<30:
            continue
        sentence=sentence.strip()
        sub_sentences=sentence.split("\n")
        for sub_sentence in sub_sentences:
            sub_sentence=sub_sentence.strip()
            if len(sub_sentence)<=20:
                continue
            ners=sw.ner_chinese(sub_sentence)
            words=jieba.cut(sub_sentence,cut_all=False)
            found_opinion=False
            list_word=[]
            for w in words:
                list_word.append(w)
                if w in opinion_words:
                    found_opinion=True
            found_person=False
            for item in ners:
                if item['type']=='PERSON':
                    found_person=True
                    break
            if found_person and found_opinion and len(list_word)>20:
                list_result.append(sub_sentence)
                # print(sub_sentence)
                # print(' '.join(list_word))
                list_seg.append(list_word)
                list_ner.append(ners)
                # print()
    return list_result,list_seg,list_ner

def add_leader_alias_names(leader_csv_path):
    list_leader_name = open(leader_csv_path, 'r', encoding='utf-8').readlines()
    list_leader_name_alias = []
    for name in list_leader_name:
        name = name.strip()
        if '·' in name:
            names = name.split("·")
            for n in names:
                if len(n) <= 1:
                    continue
                list_leader_name_alias.append(n)
            list_leader_name_alias.append(name)
        else:
            list_leader_name_alias.append(name)
    # print(list_leader_name_alias)
    for n in list_leader_name_alias:
        jieba.add_word(n)
    return list_leader_name_alias


def get_leader_opinions(text,save_path="",search_keywords_path="search_keywords.csv",leader_path="g20_leaders.csv",user_dict_paths=None,sw=None):
    if user_dict_paths!=None:
        for path in user_dict_paths:
            jieba.load_userdict(path)

    list_leader_name_alias=add_leader_alias_names(leader_path)
    if sw==None:
        sw = StanzaWrapper(auto_download_en=False, auto_download_zh=False)
    # sw.download(lang='zh',verbose=False)

    search_keywords = [w.strip() for w in open(search_keywords_path, "r", encoding='utf-8').readlines()]
    g20_leaders = [w.strip() for w in open(leader_path, "r", encoding='utf-8').readlines()]
    # print("loading dictionaries...")
    # print(search_keywords)
    # print(g20_leaders)
    # print()

    if len(text)<30:
        return []
    found_keywords=False
    for k in search_keywords:
        if k in text:
            found_keywords=True
            break
    if not found_keywords:
        return []

    list_sentence=sw.tokenize_sentence(text,lang='zh')
    list_result=[]
    for sentence in list_sentence:
        # print(sentence)
        if len(sentence)<20:
            continue
        sentence=sentence.strip()
        sub_sentences=sentence.split("\n")

        for sub_sentence in sub_sentences:
            sub_sentence=sub_sentence.strip()
            # print(sub_sentence)
            if len(sub_sentence)<=20:
                continue
            # ners=sw.ner_chinese(sub_sentence)
            # words=jieba.cut(sub_sentence,cut_all=False)
            list_word=[]
            found_a_leader=False
            found_keyword=False
            first_leader=""
            first_keyword=""
            for name in list_leader_name_alias:
                if name in sub_sentence:
                    found_a_leader=True
                    first_leader=name
                    break
            for k in search_keywords:
                if k in sub_sentence:
                    first_keyword=k
                    found_keyword=True
                    break

            if found_a_leader and found_keyword and len(sub_sentence)>20:
                # print(sub_sentence)
                model={
                    "opinion":sub_sentence,
                    "first_found_keyword":first_keyword,
                    "first_found_leader":first_leader
                }
                list_result.append(model)
                # print(' '.join(list_word))
                # print()
    if save_path!="":
        quick_save_csv(save_path,['opinion','first_found_keyword','first_found_leader'], list_result)

    return list_result